@@ -16,6 +16,7 @@ import json
 import logging
 import os
 import time
+import yaml
 
 from heatclient.common import deployment_utils
 from heatclient import exc as heat_exc
@@ -142,7 +143,7 @@ class DeployStackAction(templates.ProcessTemplatesAction):
         # check to see if the stack exists
         heat = self.get_orchestration_client(context)
         try:
-            stack = heat.stacks.get(self.container)
+            stack = heat.stacks.get(self.container, resolve_outputs=False)
         except heat_exc.HTTPNotFound:
             stack = None
 
@@ -197,11 +198,21 @@ class DeployStackAction(templates.ProcessTemplatesAction):
             except swiftexceptions.ClientException:
                 pass
             LOG.info("Perfoming Heat stack create")
-            return heat.stacks.create(**stack_args)
+            try:
+                return heat.stacks.create(**stack_args)
+            except heat_exc.HTTPException as err:
+                err_msg = "Error during stack creation: %s" % (err,)
+                LOG.exception(err_msg)
+                return actions.Result(error=err_msg)
 
         LOG.info("Performing Heat stack update")
         stack_args['existing'] = 'true'
-        return heat.stacks.update(stack.id, **stack_args)
+        try:
+            return heat.stacks.update(stack.id, **stack_args)
+        except heat_exc.HTTPException as err:
+            err_msg = "Error during stack update: %s" % (err,)
+            LOG.exception(err_msg)
+            return actions.Result(error=err_msg)
 
     def set_tls_parameters(self, parameters, env,
                            local_ca_path=constants.LOCAL_CACERT_PATH):
@@ -321,6 +332,90 @@ class DeploymentFailuresAction(base.TripleOAction):
                 'Ansible errors file not found at %s' % failures_file)
 
 
+class DeploymentStatusAction(base.TripleOAction):
+    """Get the deployment status and update it if necessary
+
+    The status will be set based off of the stack status and any running
+    config_download workflow.
+    """
+
+    def __init__(self,
+                 plan=constants.DEFAULT_CONTAINER_NAME):
+        super(DeploymentStatusAction, self).__init__()
+        self.plan = plan
+
+    def run(self, context):
+        orchestration_client = self.get_orchestration_client(context)
+        workflow_client = self.get_workflow_client(context)
+        swift_client = self.get_object_client(context)
+
+        try:
+            stack = orchestration_client.stacks.get(self.plan)
+        except heat_exc.HTTPNotFound:
+            return dict(status_update=None,
+                        deployment_status=None)
+
+        try:
+            headers, body = swift_client.get_object(
+                '%s-messages' % self.plan,
+                'deployment_status.yaml')
+
+            deployment_status = yaml.safe_load(body)['deployment_status']
+        except swiftexceptions.ClientException:
+            deployment_status = None
+
+        stack_status = stack.stack_status
+        cd_status = None
+        ansible_status = None
+        # Will get set to new status if an update is required
+        status_update = None
+
+        cd_execs = workflow_client.executions.find(
+            workflow_name='tripleo.deployment.v1.config_download_deploy')
+        cd_execs.sort(key=lambda x: x.updated_at)
+        if cd_execs:
+            cd_exec = workflow_client.executions.get(cd_execs[-1].id)
+            cd_status = cd_exec.state
+            ansible_status = json.loads(
+                cd_exec.output).get('deployment_status')
+
+        def update_status(status):
+            # If we need to update the status return it
+            if deployment_status != status:
+                return status
+
+        # Update the status if needed. We do this since tripleoclient does not
+        # yet use a single API for overcloud deployment. Since there is no long
+        # running process to make sure the status is updated, we instead update
+        # the status if needed when we get it with this action.
+        #
+        # The logic here is:
+        #
+        # If stack or config_download is in progress, then the status is
+        # deploying.
+        #
+        # Else if stack is failed or config_download is failed or ansible is
+        # failed, then the status is failed.
+        #
+        # Else if config_download status is success and ansible is success
+        # then status is success.
+        #
+        # Else, we just return the read deployment_status from earlier.
+        if stack_status.endswith('IN_PROGRESS') or cd_status == 'RUNNING':
+            status_update = update_status('DEPLOYING')
+        elif stack_status.endswith('FAILED') or cd_status == 'FAILED' \
+                or ansible_status == 'DEPLOY_FAILED':
+            status_update = update_status('DEPLOY_FAILED')
+        elif cd_status == 'SUCCESS' and ansible_status == 'DEPLOY_SUCCESS':
+            status_update = update_status('DEPLOY_SUCCESS')
+
+        return dict(cd_status=cd_status,
+                    stack_status=stack_status,
+                    deployment_status=deployment_status,
+                    ansible_status=ansible_status,
+                    status_update=status_update)
+
+
 class ConvertStatusAction(base.TripleOAction):
     """Translate a Heat stack status into a config-download deployment status
 
