@@ -72,7 +72,7 @@ class Config(object):
             if server_id in v:
                 return k
 
-    def get_config_dict(self, deployment):
+    def get_deployment_resource_id(self, deployment):
         if '/' in deployment.attributes['value']['deployment']:
             deployment_stack_id = \
                 deployment.attributes['value']['deployment'].split('/')[-1]
@@ -82,6 +82,9 @@ class Config(object):
         else:
             deployment_resource_id = \
                 deployment.attributes['value']['deployment']
+        return deployment_resource_id
+
+    def get_config_dict(self, deployment_resource_id):
         deployment_rsrc = self.client.software_deployments.get(
             deployment_resource_id)
         config = self.client.software_configs.get(
@@ -135,12 +138,24 @@ class Config(object):
 
     def initialize_git_repo(self, dirname):
         repo = git.Repo.init(dirname)
+        gitignore_path = os.path.join(dirname, '.gitignore')
+
+        # Ignore tarballs, which we use for the export process
+        if not os.path.exists(gitignore_path):
+            with open(gitignore_path, 'w') as f:
+                f.write('*.tar.gz\n')
+            # For some reason using repo.index.add is not working, so go
+            # directly to the GitCmd interface.
+            repo.git.add('.gitignore')
+
         return repo
 
     def snapshot_config_dir(self, repo, commit_message):
         if repo.is_dirty(untracked_files=True):
             self.log.info('Snapshotting {}'.format(repo.working_dir))
-            repo.index.add('*')
+            # Use repo.git.add directly as repo.index.add defaults to forcing
+            # commit of ignored files, which we don't want.
+            repo.git.add('.')
             commit = repo.index.commit(commit_message)
             self.log.info('Created commit {}'.format(commit.hexsha))
         else:
@@ -185,6 +200,8 @@ class Config(object):
     def write_config(self, stack, name, config_dir, config_type=None):
         # Get role data:
         role_data = self.stack_outputs.get('RoleData', {})
+        role_group_vars = self.stack_outputs.get('RoleGroupVars', {})
+        allnodes_vars = self.stack_outputs.get('AllNodesConfig', {})
         for role_name, role in six.iteritems(role_data):
             role_path = os.path.join(config_dir, role_name)
             self._mkdir(role_path)
@@ -242,8 +259,30 @@ class Config(object):
         server_roles = {}
 
         for deployment in deployments_data:
+            # Check if the deployment value is the resource name. If that's the
+            # case, Heat did not create a physical_resource_id for this
+            # deployment since it does not trigger on this stack action. Such
+            # as a deployment that only triggers on DELETE, but this is a stack
+            # create. If that's the case, just skip this deployment, otherwise
+            # it will result in a Not found error if we try and query the
+            # deployment API for this deployment.
+            dep_value_resource_name = deployment.attributes[
+                'value'].get('deployment') == 'TripleOSoftwareDeployment'
+
+            # if not check the physical_resource_id
+            if not dep_value_resource_name:
+                deployment_resource_id = self.get_deployment_resource_id(
+                    deployment)
+
+            if dep_value_resource_name or not deployment_resource_id:
+                warnings.warn('Skipping deployment %s because it has no '
+                              'valid uuid (physical_resource_id) '
+                              'associated.' %
+                              deployment.physical_resource_id)
+                continue
+
             server_id = deployment.attributes['value']['server']
-            config_dict = self.get_config_dict(deployment)
+            config_dict = self.get_config_dict(deployment_resource_id)
 
             # deployment_name should be set via the name property on the
             # Deployment resources in the templates, however, if it's None
@@ -294,9 +333,23 @@ class Config(object):
                 [i for i in config_dict['inputs']
                  if i['name'] == 'deploy_server_id'].pop()
             deploy_server_id_input['value'] = server_id
-            server_deployments.setdefault(
-                server_names[server_id],
-                []).append(config_dict)
+
+            # We don't want to fail if server_id can't be found, as it's
+            # most probably due to blacklisted nodes. However we fail for
+            # other errors.
+            try:
+                server_deployments.setdefault(
+                    server_names[server_id],
+                    []).append(config_dict)
+            except KeyError:
+                self.log.warning('Server with id %s is ignored from config '
+                                 '(may be blacklisted)' % server_id)
+                # continue the loop as this server_id is probably excluded
+                continue
+            except Exception as err:
+                err_msg = ('Error retrieving server name from this server_id: '
+                           '%s with this error: %s' % server_id, err)
+                raise Exception(err_msg)
 
             role = self.get_role_from_server_id(stack, server_id)
             role_deployments = role_deployment_names.setdefault(role, {})
@@ -328,6 +381,13 @@ class Config(object):
 
         group_vars_dir = os.path.join(config_dir, 'group_vars')
         self._mkdir(group_vars_dir)
+        # Write allnodes_vars to 'overcloud' group as it doesn't apply
+        # to the undercloud, and 'overcloud' is the inventory group
+        # for all overcloud nodes, regardless of the --stack name.
+        filepath = os.path.join(group_vars_dir, 'overcloud.yaml')
+        with self._open_file(filepath) as vars_file:
+            yaml.safe_dump(allnodes_vars, vars_file, default_flow_style=False)
+
         host_vars_dir = os.path.join(config_dir, 'host_vars')
         self._mkdir(host_vars_dir)
 
@@ -372,12 +432,12 @@ class Config(object):
                                "config-download." % d['deployment_name'])
                     warnings.warn(message, DeprecationWarning)
 
-                with open(deployment_path, 'w') as f:
+                with open(deployment_path, 'wb') as f:
                     template_data = deployment_template.render(
                         deployment=d,
                         server_id=server_ids[server])
                     self.validate_config(template_data, deployment_path)
-                    f.write(template_data)
+                    f.write(template_data.encode('utf-8'))
 
         for role, deployments in role_deployment_names.items():
             group_var_role_path = os.path.join(group_vars_dir, role)
@@ -387,7 +447,8 @@ class Config(object):
                 template_data = group_var_role_template.render(
                     role=role,
                     pre_deployments=deployments['pre_deployments'],
-                    post_deployments=deployments['post_deployments'])
+                    post_deployments=deployments['post_deployments'],
+                    role_group_vars=role_group_vars[role])
                 self.validate_config(template_data, group_var_role_path)
                 f.write(template_data)
 
